﻿using System;
using System.Collections.Generic;
using System.Text;

namespace Uno.UI.Samples.Controls
{

	[AttributeUsage(AttributeTargets.Class, Inherited = false, AllowMultiple = true)]
	public sealed class SampleControlInfoAttribute : SampleAttribute
	{
		public SampleControlInfoAttribute(
			string category = null,
			string controlName = null,
			Type viewModelType = null,
			bool ignoreInSnapshotTests = false,
			string description = null)
			: base(category)
		{
			Name = controlName;
			ViewModelType = viewModelType;
			IgnoreInSnapshotTests = ignoreInSnapshotTests;
			Description = description;
		}
	}

	[AttributeUsage(AttributeTargets.Class, Inherited = false, AllowMultiple = false)]
	public class SampleAttribute : Attribute
	{
		/// <summary>
		/// Marks a class as a sample test control that can be browsed by the SampleChooserControl
		/// and which can be used by automated tests.
		/// </summary>
		/// <param name="categories">An optional list of categories to which this sample is related to</param>
		public SampleAttribute(params string[] categories)
		{
			Categories = categories;
		}

		/// <summary>
		/// An optional list of categories to which this sample is related to
		/// </summary>
		public string[] Categories { get; }

		/// <summary>
		/// An optional name for this sample. Default will be the name of the class.
		/// </summary>
		public string Name { get; set; }

		/// <summary>
		/// Determines if this sample should be included or not in the automated screenshot comparison.
		/// If this flag is not set, the sample will be included.
		/// </summary>
		public bool IgnoreInSnapshotTests { get; set; }

		/// <summary>
		/// An optional ViewModel type that will be instantiated and set as DataContext of the sample control
		/// </summary>
		public Type ViewModelType { get; set; }

		/// <summary>
		/// An optional description of the sample. A good practice is to explain the expected result of the sample.
		/// </summary>
		public string Description { get; set; }
	}
}
